<?php

namespace Drupal\one_time_password;

use Drupal\Component\Utility\Crypt;
use Drupal\Core\PrivateKey;
use Drupal\Core\Site\Settings;
use Drupal\user\UserInterface;

/**
 * Provides methods for logging in users.
 *
 * @internal
 */
trait OtpLoginTrait {

  /**
   * The private key service.
   */
  protected ?PrivateKey $privateKey = NULL;

  /**
   * Generate a hash that can uniquely identify an account's state.
   */
  protected function getLoginHash(UserInterface $account): string {
    // Using account login will mean this hash will become invalid once user has
    // authenticated via TFA.
    $data = implode(':', [
      $account->getAccountName(),
      $account->getPassword(),
      $account->getLastLoginTime(),
    ]);
    $key = $this->getPrivateKey()->get() . Settings::get('hash_salt');
    return Crypt::hmacBase64($data, $key);
  }

  /**
   * Get the `private_key` service.
   */
  protected function getPrivateKey(): PrivateKey {
    if ($this->privateKey === NULL) {
      $this->setPrivateKey(\Drupal::service('private_key'));
    }
    return $this->privateKey;
  }

  /**
   * Set the `private_key` service.
   */
  protected function setPrivateKey(PrivateKey $private_key): void {
    $this->privateKey = $private_key;
  }

}
